#!/bin/sh
#/*
# * Copyrighy (c) 1999 CLE Project. All rights reserved.
# * Copyright (c) 1999 Yuan-Chung Cheng. All rights reserved.
# *
# *   Modified by Chih-Wei Huang         January 6, 2000
# *
# * Redistribution and use in source and binary forms, with or without
# * modification, are permitted provided that the following conditions
# * are met:
# * 1. Redistributions of source code must retain the above copyright
# *    notice, this list of conditions and the following disclaimer.
# * 2. Redistributions in binary form must reproduce the above copyright
# *    notice, this list of conditions and the following disclaimer in the
# *    documentation and/or other materials provided with the distribution.
# * 3. All advertising materials mentioning features or use of this software
# *    must display the following acknowledgement:
# *      This product includes software developed by
# *      Yuan-Chung Cheng and contributors for CLE Project
# * 4. Neither the name of the Author nor the names of contributors
# *    may be used to endorse or promote products derived from this software
# *    without specific prior written permission.
# *
# * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# * SUCH DAMAGE.
# */

# The system depended variables...

VERSION=0.9

DefaultFontPath=/usr/share/fonts/ttf
ScriptDir=/usr/share/fonts/install
ScriptSubfix="ttfm"

# USAGE

Usage()
{
    echo " " > /dev/stderr
    echo "True-Type Font Manager $VERSION" > /dev/stderr
    echo "       " > /dev/stderr
    echo "Usage: $0 [option]" > /dev/stderr
    echo "       " > /dev/stderr
    echo "       --add <file>...    install ttf font" > /dev/stderr
    echo "       --remove <file>... remove ttf font from the system" > /dev/stderr
    echo "       --list <module>... list all ttf fonts on the system" > /dev/stderr
    echo "       --modules          list all ttf manager modules on the system" > /dev/stderr
    echo "       --setdefault <module> <file> " > /dev/stderr
    echo "                          set default font of module to file" > /dev/stderr
    echo "       --initm <module>.. initialize modules" > /dev/stderr
    echo "       --help             show this info" > /dev/stderr
}

# check the system requirement...
check_system()
{
    # ttfinfo

    TTFINFO=`type -path ttfinfo`
    if [ "z$TTFINFO" = "z" ] ; then
	echo "Can't find ttfinfo, please install ttfinfo first." > /dev/stderr
	exit
    fi

    # ScriptDir
    
    if [ ! -d $ScriptDir ] ; then
	echo "Script directory is incorrect! " > /dev/stderr
	echo "Please check out your system setup." > /dev/stderr
	exit
    fi

    # DefaultFontPath
    if [ ! -d $DefaultFontPath ] ; then
	echo "Directory $DefaultFontPath is not found, create it! " > /dev/stderr
	mkdir -p $DefaultFontPath
    fi
    
    # install scripts
    
    INSTALLSCRIPT=`/bin/ls $ScriptDir/*.${ScriptSubfix}`
    
    if [ "z$INSTALLSCRIPT" = "z" ] ; then
	echo "Can't find any install scripts, do nothing!!" > /dev/stderr
	exit 0
    fi
}

list_fonts()
{
    check_system;

    # $@ modules list.
    if [ $# -lt 1 ] ; then
        MODLIST="$INSTALLSCRIPT"
    else
        MODLIST="$@"
    fi

    for i in $MODLIST ; do
        modulename=`basename $i .${ScriptSubfix}`
        MOD="${ScriptDir}/${modulename}.${ScriptSubfix}"
        if [ -x "$MOD" ] ; then
            # run script
            $MOD --name
	    $MOD --list
	else
            echo "$MOD is not executable, ignore it!!" > /dev/stderr
	fi
    done
}

list_modules()
{
    check_system;

    for i in $INSTALLSCRIPT ;
    do
        if [ -x $i ] ; then
            # module name
	    echo `basename $i .${ScriptSubfix}` 
            #$i --name
	else
            echo "$i is not executable, ignore it!!" > /dev/stderr
	fi
    done
}

setdefault_fonts()
{

    if [ $# -lt 2 ] ; then
        echo "too few arguments!!" > /dev/stderr
	Usage;
	exit 1;
    fi

    check_system;

    # $1=module  $2=fontfile

    modulename=`basename $1 .${ScriptSubfix}`
    fontfile=`basename $2`
    
    # is this module exist ??
    if [ ! -x "${ScriptDir}/${modulename}.${ScriptSubfix}" ] ; then
        echo "Module $1 is not found!!" > /dev/stderr
	exit 1
    fi

    #check whether this font has installed for this module
    
    i=`${ScriptDir}/${modulename}.${ScriptSubfix} --list | grep -e "$fontfile"`
    if [ -z "$i" ] ; then
        echo "Font $fontfile is not installed in module $1!!" > /dev/stderr
	exit 1
    fi

    # set the default font... :p
    ${ScriptDir}/${modulename}.${ScriptSubfix} --setdefault $fontfile
}

add_fonts()
{

    check_system;
    # Run for every fontfiles...
    for FONTFILE in $* ;
    do
    
	FONTFILENAME=`basename $FONTFILE`
	FONTFILEPATH=`dirname $FONTFILE`
	
	# copy the fontfile to the correct place
	if [ ! -e "${DefaultFontPath}/${FONTFILENAME}" ] ; then
	    echo "Copying $FONTFILE to ${DefaultFontPath}/${FONTFILENAME}"
	    cp -f $FONTFILE ${DefaultFontPath}/${FONTFILENAME} || {
		    echo "Failed in copying $FONTFILE"
		    exit 1
	    }
	else
	    i=`/bin/ls -i $FONTFILE | tr -d " " | cut -d"/" -f1`
	    j=`/bin/ls -i ${DefaultFontPath}/${FONTFILENAME} | tr -d " " | \
			    cut -d"/" -f1`
	    if [ "$i" = "$j" -a "z$i" != "z" ] ; then
		echo "${FONTFILENAME} already in the ${DefaultFontPath}, no copying needed." > /dev/stderr
	    else
		echo "Already have ${DefaultFontPath}/${FONTFILENAME}, ignore it!!" > /dev/stderr
		continue;
	    fi
	fi
	
	# get and save the font information
	FONTINFOS=`$TTFINFO -c $FONTFILE | tr ' ' '?' `
	
	if [ "z$FONTINFOS" = "z" ] ; then
	    echo "Format of $FONTFILE is not correct, ignore it!!" > /dev/stderr
	    continue;
	fi
	
	# OK!!
	# export info as environment variable in a subshell
	(
	
	# export the information
	for i in $FONTINFOS ;
	do
	    i=`echo $i |  tr '?' ' ' `
	    eval "export $i"
	done
	
	# do the installation
	for i in $INSTALLSCRIPT ;
	do
	    if [ -x $i ] ; then
		# run script
		$i --add ${DefaultFontPath}/${FONTFILENAME} || {
			echo "Error in running $i to add ${DefaultFontPath}/${FONTFILENAME}!!" > /dev/stderr
			}
	    else
		echo "$i is not executable, ignore it!!" > /dev/stderr
		continue;
	    fi
	done
	)
	
    done
}


remove_fonts()
{

    check_system;

    # Run for every fontfiles...
    for FONTFILE in $* ;
    do
    
	FONTFILENAME=`basename $FONTFILE`

	# is this file exist ??
	if [ ! -e "${DefaultFontPath}/${FONTFILENAME}" ] ; then
	    echo "${FONTFILENAME} is not exist in ${DefaultFontPath}." > /dev/stderr
	    continue;
	fi
	
	# call script with --remove
	for i in $INSTALLSCRIPT ;
	do
	    if [ -x $i ] ; then
		# run script
		$i --remove ${DefaultFontPath}/${FONTFILENAME} || {
			echo "Error in running $i to remove ${DefaultFontPath}/${FONTFILENAME}!!" > /dev/stderr
			}
	    else
		echo "$i is not executable, ignore it!!" > /dev/stderr
		continue;
	    fi
	done

      # set the variable KEEP_FONT if you don't want to remove the ttf file
        [ "z$KEEP_FONT" = "z" ] && rm -f ${DefaultFontPath}/${FONTFILENAME}
    done
}

init_modules()
{
    if [ $# -lt 1 ] ; then
        echo "too few arguments!!" > /dev/stderr
        Usage;
        exit 1;
    fi

    check_system;

    # $@ modules list. if $1 = all, initialize all modules
    # hope there is no modules named 'all' :p

    if [ "$1" = "all" ]; then
        MODLIST="$INSTALLSCRIPT"
    else
        MODLIST="$@"
    fi

    for m in $MODLIST ; do
        modulename=`basename $m .${ScriptSubfix}`
        # is this module exist ??
        MOD="${ScriptDir}/${modulename}.${ScriptSubfix}"
        if [ ! -x "$MOD" ] ; then
            echo "Module $m is not found, ignored!" > /dev/stderr
            continue
        fi

        FONTLIST=`/bin/ls ${DefaultFontPath}/*.tt?`
        for FONTFILE in $FONTLIST ; do
            FONTFILENAME=`basename $FONTFILE`
            FONTFILEPATH=`dirname $FONTFILE`

          # check whether this font has installed for this module
            i=`$MOD --list | grep -e "$FONTFILENAME" `
            [ -z "$i" ] || continue

            # get and save the font information
            FONTINFOS=`$TTFINFO -c $FONTFILE | tr ' ' '?' `

          # ignore file that is not ttf
            [ "z$FONTINFOS" = "z" ] && continue

            # export info as environment variable in a subshell
            (

            # export the information
            for i in $FONTINFOS ; do
                i=`echo $i |  tr '?' ' ' `
                eval "export $i"
            done

            $MOD --add ${DefaultFontPath}/${FONTFILENAME} || {
                echo "Error in running $i to add ${DefaultFontPath}/${FONTFILENAME}!!" > /dev/stderr
             }

            )

        done
    done
}

# DONE!!

## Start of main program

if [ "$#" = "0" ] ; then
    Usage;
    exit 1;
fi

MODE=$1
shift 1

case $MODE in 
--help)
    Usage;
    exit 0;
    ;;
--list)
    list_fonts $@;
    exit 0;
    ;;
--modules)
    list_modules;
    exit 0;
    ;;
--add)
    add_fonts $@;
    exit 0;
    ;;
--remove)
    remove_fonts $@;
    exit 0;
    ;;
--setdefault)
    setdefault_fonts $@;
    exit 0;
    ;;
--initm)
    init_modules $@;
    exit 0;
    ;;
esac

# wrong option,, show usage info...
Usage;
exit 1

# ChangeLog
#
# $Log: ttfm.sh,v $
# Revision 1.4  2000/02/13 10:32:32  cwhuang
# my source
#
# Revision 1.2  2000/02/10 11:40:14  platin
# Log entry was added.
#
#
